$def with ()
$var title: Site Owner

<form method=post>
<div><label>Your address<br>
<input type=text readonly value="$tx.host.owner['uid'][0]"></div>
<div><label>Name<br>
<input type=text name=name value="$tx.host.owner['name'][0]"></div>
<div><label>Note<br>
<input type=text name=note value="$tx.host.owner.get('note', [''])[0]"></div>
<button>Update</button>
</form>
